<?php

require 'autoload.php';

$cli = eZCLI::instance();
$ini = eZINI::instance();
$contentINI = eZINI::instance( 'content.ini' );
$script = eZScript::instance( array( 'description' => ( "eZ Publish Shell\n\n" .
                                                        "Allows to type command to manipulate eZ Publish content\n" .
                                                        "\n" .
                                                        "ezshell.php --username=user" ),
                                     'use-session' => true,
                                     'use-modules' => true,
                                     'use-extensions' => true ) );

$script->startup();
$options = $script->getOptions( "[password:][user:]",
                                "",
                                array( 'password' => 'Password, default is empty',
                                       'user'     => 'User login, default is anonymous' ) );
$script->initialize();

$userLogin = 'anonymous';
$userPassword = false;
$siteaccess = false;
$verbose = false;


function changeSiteAccessSetting( &$siteaccess, $optionData )
{
    if ( file_exists( 'settings/siteaccess/' . $optionData ) )
    {
        $siteaccess = $optionData;
    }
    elseif ( isExtensionSiteaccess( $optionData ) )
    {
        $siteaccess = $optionData;
        eZExtension::prependExtensionSiteAccesses( $siteaccess );
    }
}

function isExtensionSiteaccess( $siteaccessName )
{
    $ini = eZINI::instance();
    $extensionDirectory = $ini->variable( 'ExtensionSettings', 'ExtensionDirectory' );
    $activeExtensions = $ini->variable( 'ExtensionSettings', 'ActiveExtensions' );

    foreach ( $activeExtensions as $extensionName )
    {
        $possibleExtensionPath = $extensionDirectory . '/' . $extensionName . '/settings/siteaccess/' . $siteaccessName;
        if ( file_exists( $possibleExtensionPath ) )
            return true;
    }
    return false;
}

if ( isset( $options['user'] ) )
{
    $userLogin = $options['user'];
}
if ( isset( $options['password'] ) )
{
    $userPassword = $options['password'];
}
if ( isset( $options['siteaccess'] ) )
{
    changeSiteAccessSetting( $siteaccess, $options['siteaccess'] );
}
if ( isset( $options['verbose'] ) )
{
    $verbose = true;
}

if ( $siteaccess === false )
{
    $siteaccess = $ini->variable( 'SiteSettings', 'DefaultAccess' );
}


if ( $userLogin == 'anonymous'
        && $ini->variable( 'SiteAccessSettings', 'RequireUserLogin' ) == 'true' )
{
    $cli->error( 'Siteaccess ' . $siteaccess . ' requires login' );
    $script->shutdown( 1 );
}

$output = new ezcConsoleOutput();
$output->formats->headline->style = array( 'bold' );
$output->formats->headline->color = 'blue';
$output->formats->headline2->style = array( 'bold' );
$output->formats->error->style = array( 'bold' );
$output->formats->error->color = 'red';


$user = eZUser::loginUser( $userLogin, $userPassword );

$question = new ezcConsoleQuestionDialog( $output );
$question->options->showResults = false; 
if ( $user === false && $userPassword === false )
{
    // ask for password
    $question->options->text = "Password:";
    $userPassword = ezcConsoleDialogViewer::displayDialog( $question );
    $user = eZUser::loginUser( $userLogin, $userPassword );
}

if ( $user === false )
{
    $cli->error( 'Can\'t login, wrong password ?' );
    $script->shutdown( 1 );
}

$access = $user->hasAccessTo( 'user', 'login' );

$hasAccess = false;
if ( $access['accessWord'] == 'no' )
{
    $hasAccess = false;
}
else if ( $access['accessWord'] == 'yes' )
{
    $hasAccess = true;
}
else
{
    // limited access
    $crc32 = eZSys::ezcrc32( $siteaccess );
    foreach( $access['policies'] as $policy )
    {
        if ( isset( $policy['SiteAccess'] ) )
        {
            if ( in_array( $crc32, $policy['SiteAccess'] ) )
            {
                $hasAccess = true;
                break ;
            }
        }
    }
}

if ( !$hasAccess )
{
    $cli->error( 'You don\'t have right to access siteaccess "' . $siteaccess . '"' );
    $script->shutdown( 1 );
}

$promptBase = $userLogin . '@' . $siteaccess;
$rootNodeID = $contentINI->variable( 'NodeSettings', 'RootNode' );
$node = eZContentObjectTreeNode::fetch( $rootNodeID );


$cli->output( 'Type \'help\' to get the list of available commands' );
$shell = new eZShell( $node, $node );

$prompt = $promptBase . ' ' . $shell->path . '$';
while ( true )
{
    $question->options->text = $prompt;
    $command = ezcConsoleDialogViewer::displayDialog( $question );
    $commandArray = explode( ' ', $command );
    $shell->execute( $output, $commandArray );
    $prompt = $promptBase . ' ' . $shell->path . '$';
}


$script->shutdown();
